% Performs alernating regression on the whole S-System defined in ToyNetwork.m
%
% alternating regression is performed as published in
% Chou I, Martens H, Voit EO: Parameter estimation in biochemical systems models with alternating regression. Theor Biol Med Model 2006, 3:25.
%
% Input:
%   X1:         time course data of X1
%   X2:         time course data of X2
%   X3:         time course data of X3
%   X4:         time course data of X4
%   Si1:        (estimated) slope of X1
%   Si2:        (estimated) slope of X2
%   Si3:        (estimated) slope of X3
%   Si4:        (estimated) slope of X4
%   bd1:        initial estimations for the parameters of the degradation term of X1
%   bd2:        initial estimations for the parameters of the degradation term of X2
%   bd3:        initial estimations for the parameters of the degradation term of X3
%   bd4:        initial estimations for the parameters of the degradation term of X4
%
% Output:
%   Production:     estimations for the parameters of the production term
%   Degradation:    estimations for the parameters of the degradation term
%   SSE:            sum of squared errors
%   Iterations:     number of iterations run
%
% Wolfgang Beyer, 2007

function [Production, Degradation, SSE, Iterations] = X1to4C(X1, X2, X3, X4, Si1, Si2, Si3, Si4, bd1, bd2, bd3, bd4)

%Generate matrices of logarithms of regressors
TSLength=length(X1);
Lp1=[ones(TSLength,1) log(X3)];
Ld1=[ones(TSLength,1) log(X1)];
Lp2=[ones(TSLength,1) log(X1)];
Ld2=[ones(TSLength,1) log(X2)];
Lp3=[ones(TSLength,1) log(X2)];
Ld3=[ones(TSLength,1) log(X3) log(X4)];
Lp4=[ones(TSLength,1) log(X1)];
Ld4=[ones(TSLength,1) log(X4)];

%Compute matrices Cp and Cd
Cp1=inv(Lp1'*Lp1)*Lp1';
Cp2=inv(Lp2'*Lp2)*Lp2';
Cp3=inv(Lp3'*Lp3)*Lp3';
Cp4=inv(Lp4'*Lp4)*Lp4';
Cd1=inv(Ld1'*Ld1)*Ld1';
Cd2=inv(Ld2'*Ld2)*Ld2';
Cd3=inv(Ld3'*Ld3)*Ld3';
Cd4=inv(Ld4'*Ld4)*Ld4';

%Iterate the actual regression
SSE=1;
Iterations=0;
while (SSE>-15) && (Iterations<100000)

    %Degradation side
    %do regression
    yd1=[log(Si1+bd1(1,1)*X1.^bd1(2,1))];
    yd2=[log(Si2+bd2(1,1)*X2.^bd2(2,1))];
    yd3=[log(Si3+bd3(1,1)*X3.^bd3(2,1).*X4.^bd3(3,1))];
    yd4=[log(Si4+bd4(1,1)*X4.^bd4(2,1))];
    
    bp1=Cp1*yd1;
    bp2=Cp2*yd2;
    bp3=Cp3*yd3;
    bp4=Cp4*yd4;

    %calculate error
    SSE=sum((yd1-Lp1*bp1).^2+(yd2-Lp2*bp2).^2+(yd3-Lp3*bp3).^2+(yd4-Lp4*bp4).^2);
    
    %transform parameters from logarithmic space in to linear space
    bp1(1,1)=exp(bp1(1,1));
    bp2(1,1)=exp(bp2(1,1));
    bp3(1,1)=exp(bp3(1,1));
    bp4(1,1)=exp(bp4(1,1));

    %Production side
    %do regression
    yp1=[log(bp1(1,1)*X3.^bp1(2,1)-Si1)];
    yp2=[log(bp2(1,1)*X1.^bp2(2,1)-Si2)];
    yp3=[log(bp3(1,1)*X2.^bp3(2,1)-Si3)];
    yp4=[log(bp4(1,1)*X1.^bp4(2,1)-Si4)];
    
    bd1=Cd1*yp1;
    bd2=Cd2*yp2;
    bd3=Cd3*yp3;
    bd4=Cd4*yp4;
    
    %calculate error
    SSE=log(SSE+sum((yp1-Ld1*bd1).^2+(yp2-Ld2*bd2).^2+(yp3-Ld3*bd3).^2+(yp4-Ld4*bd4).^2));
    
    %transform parameters from logarithmic space in to linear space
    bd1(1,1)=exp(bd1(1,1));
    bd2(1,1)=exp(bd2(1,1));
    bd3(1,1)=exp(bd3(1,1));
    bd4(1,1)=exp(bd4(1,1));

    Iterations=Iterations+1;
end

%show theoretical values together with the calculated ones
Production=[[12; -.8; 8; .5; 3; .75; 2; .5] [bp1; bp2; bp3; bp4]];
Degradation=[[10; .5; 3; .75; 5; .5; .2; 6; .8] [bd1; bd2; bd3; bd4]];